
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MODULE     : mmx-mode.el
;; DESCRIPTION: A Mathemagix mode for emacs
;; COPYRIGHT  : (C) 2004  Enrique A. Tobis (etobis@dc.uba.ar)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Usage: put the following code in your .emacs file after setting the path
;; (setq auto-mode-alist
;;      (append auto-mode-alist
;;	      '(("\\.mmx" . mmx-mode)
;;		("\\.mmh" . mmx-mode))))
;; (autoload 'mmx-mode
;;   "[the correct path]/mmx-mode.el"
;;   "Major mode for editing Mathemagix files" t)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This software falls under the GNU general public license and comes
;; WITHOUT ANY WARRANTY WHATSOEVER. See the file LICENSE for more details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; So far, the only functionality provided is keyword highlighting

;; Customization

;; This is the list of Mathemagix keywords. Yo may add or remove any of them
(defvar mmx-keywords 
  (list "and" "begin" "break" "case" "continue" "div" "else"
    "error" "exists" "export" "forall" "for"
    "generate" "if" "import" "join" "lambda"
    "load" "loop" "literal" "mod" "obtain" "operator"
    "or" "postfix" "retrieve" "return" "split" "step"
    "then" "until" "use" "while" "yield")
  )

;; We define the hook for this major-mode, so that people can add
;; initialization code
(defvar mmx-mode-hook nil)

;; This is the main function. It loads Mathemagix mode
(defun mmx-mode ()
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'mmx-mode) 
  (setq mode-name "Mathemagix") ;This sets the label in the Mode Line
  (mmx-define-keywords)
  (run-hooks 'mmx-mode-hook)
  )

;; This function sets up the variable font-lock-defaults, which tells
;; the font-lock minor mode how to do highlighting.
(defun mmx-define-keywords ()
  "This function defines all the elements of the list mmx-keywords as keywords for font-locking"
  (setq font-lock-defaults '((regexp-opt mmx-keywords) nil nil nil))
  )

;; Administrative stuff
(defconst mmx-version "0.0.1"
  "mathemagix-mode version number")
(defun mmx-version ()
  "Echo the current version of mathemagix-mode in the minibuffer."
  (interactive)
  (message "mmx-mode version %s" mmx-version)
)

;; We make ourselves available
(provide 'mmx-mode)
