
/******************************************************************************
* MODULE     : ext_concrete.cc
* DESCRIPTION: Generate extension code for concrete exported types
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <extend.hh>
#include <ccpp.hh>
#include <pretty.hh>
#include <converter.hh>
namespace mmx {

static converter cvt;

/******************************************************************************
* Filtering declarations
******************************************************************************/

static generic
tmp_hack_forall (generic g) {
  if (is_func (g, MMX_FORALL, 2) &&
      is_func (g[1], MMX_TYPE, 2))
    return Forall (g[1][1], tmp_hack_forall (g[2]));
  if (is_list (g) && (!nil (g)))
    return cons (tmp_hack_forall (car (g)),
		 tmp_hack_forall (cdr (g)));
  return g;
}

list
extender::filter (list l, list target_l) {
  // keep only those declarations which are exported
  if (nil (l)) return l;
  else {
    bool ok= false;
    generic var  = get_decl_var  (car (l));
    generic type = get_decl_type (car (l));
    if (export_env -> contains (var)) {
      generic stype= convert_prepare (tmp_hack_forall (export_env [var]));
      generic dtype= convert_prepare (tmp_hack_forall (type));
      proof p= cvt->convert (stype, dtype, CV_NORMAL);
      ok= !nil (p);
    }
    if (ok) return cons (car (target_l), filter (cdr (l), cdr (target_l)));
    else return filter (cdr (l), cdr (target_l));
  }
}

/******************************************************************************
* Substitute class arguments of templates by corresponding geninst_nr types
******************************************************************************/

list
extender::class_to_geninst (list l, int i) {
  if (nil (l)) return l;
  if (car (l) == MMX_CLASS)
    return cons (geninst (i), class_to_geninst (cdr (l), i+1));
  else return cons (car (l), class_to_geninst (cdr (l), i));
}

/******************************************************************************
* Produce the generic code for a given type
******************************************************************************/

list
extender::contype_tmpl_args (list l, int kind, int i, int n) {
  if (nil (l)) return l;
  symbol  var1= string ("x_") * as_string (i);
  symbol  var2= string ("y_") * as_string (i);
  symbol  var3= string ("save_") * as_string (n);
  list code;
  generic type= car (l), tp;
  if (type == MMX_CLASS) tp= gentype;
  else tp= type_as_ccpp (type);
  switch (kind) {
  case 1: // fields of type structure 
    code= Type (var1, tp);
    break;
  case 2: // arguments for constructor type structure
    code= Type (var2, tp);
    break;
  case 3: // initializers for fields of type structure
    code= generic (list (var1, var2));
    break;
  case 4: // for calling new operator
    code= generic (var1);
    break;
  case 5: // save old generic type arguments
    if (type == MMX_CLASS)
      code= Assign (Type (var3, gentype), curtype (n));
    break;
  case 6: // set generic type arguments
    if (type == MMX_CLASS)
      code= Assign (curtype (n), var1);
    break;
  case 7: // recover old generic type arguments
    if (type == MMX_CLASS)
      code= Assign (curtype (n), var3);
    break;
  }
  if (type == MMX_CLASS) n++;
  return code * contype_tmpl_args (cdr (l), kind, i+1, n);
}

list
extender::contype_cast_args (generic ccpp_id, list l, int i) {
  if (nil (l)) return l;
  else {
    generic first= parameter (i);
    generic stype= car (l);
    generic dtype= substitute (stype, geninst (), ccpp_id);
    if (dtype != stype) {
      generic cvt= Specialize (symbol ("convert"), dtype, stype);
      first= list (cvt, parameter (i));
    }
    return cons (first, contype_cast_args (ccpp_id, cdr (l), i+1));
  }
}

void
extender::contype_routine (
  generic ccpp_id, generic ccpp_type, list args, generic g, generic h)
{
  // cout << "Generate method " << g << " for " << ccpp_type << lf;
  if (!(is_func (h, MMX_DEFINE, 2) &&
	is_func (g, MMX_TYPE, 2) &&
	is_symbol (g[1]) &&
	is_func (g[2], MMX_FUNCTION, 2) &&
	is_func (g[2][1], MMX_CROSS)))
    error (g, "Assertion failed in contype_routine");
  else {
    // main computation
    generic rt= g[2][2];
    if (is_func (rt, MMX_CROSS, 0)) rt= MMX_VOID;
    list l1= cdr (as_list (g[2][1]));
    list l2= contype_cast_args (ccpp_type, l1, 1);
    generic op  = ccpp_back (h[2]);
    if (op == symbol ("new")) op= ccpp_type_protect (ccpp_type);
    generic expr= cons (op, l2);
    generic st  = substitute (rt, geninst (), ccpp_type);
    generic cv  = Specialize (symbol ("convert"), rt, st);
    if (rt != st) expr= list (cv, expr);

    // save generic types if necessary and treat void return values
    list code;
    list pre = contype_tmpl_args (args, 5) * contype_tmpl_args (args, 6);
    list post= contype_tmpl_args (args, 7);
    if (nil (pre) && nil (post)) {
      if (rt == MMX_VOID) code= list (expr);
      else code= list (Return (expr));
    }
    else {
      if (rt == MMX_VOID) code= pre * list (expr) * post;
      else {
	generic rvar= Type (symbol ("ret"), rt);
	generic comp= Assign (rvar, expr);
	code= pre * list (comp) * post * list (Return (symbol ("ret")));
      }
    }

    // generate declaration
    generic abbr_type_rep= contype_rep (ccpp_id);
    generic var = Scope (abbr_type_rep, g[1]);
    generic type= name_parameters (g[2]);
    generate (Define (Type (var, type), Begin (code)));
  }
}

void
extender::contype_produce (
  generic mmx_id, generic ccpp_id, list args, int i, bool atomic_flag)
{
  // cout << "Produce " << mmx_id << ", " << ccpp_id << ", " << args << lf;
  generic abbr_type= contype (ccpp_id);
  generic abbr_type_rep= contype_rep (ccpp_id);
  generic mmx_type = mmx_id;
  generic ccpp_type= ccpp_id;

  // Pretreatment of template arguments
  list mmx_args = class_to_geninst (args);
  list ccpp_args= type_as_ccpp (mmx_args);
  if (!nil (args)) {
    mmx_type = cons (mmx_id , mmx_args);
    ccpp_type= cons (ccpp_id, ccpp_args);
  }
  list tl1= contype_tmpl_args (args, 1);
  list tl2= contype_tmpl_args (args, 2);
  list tl3= contype_tmpl_args (args, 3);
  list tl4= contype_tmpl_args (args, 4);

  // Produce the prototype
  generic ct= Define (Type (abbr_type_rep, Inline (Constructor (tl2))),
		      Initialize (cons (list (gentype_rep), tl3), Begin ()));
  generic in= Define (Type (symbol ("internal"),
			    Function (MMX_VOID, MMX_BOOL)),
		      Begin (Return (symbol ("true"))));
  list r= filter (specialize (gen_imports, mmx_type, false),
		  specialize (gen_imports, geninst (), false));
  list l= decl_as_ccpp (r, true);
  list mbs= tl1 * list (ct, in) * l;
  generate (make_struct (abbr_type_rep, mbs, list (gentype_rep)));

  // Produce the constructor
  generic ft  = Function (Cross (tl1), gentype);
  generic fun = Type (abbr_type, ft);
  generic ptr = New (abbr_type_rep, tl4);
  generic body= Begin (Return (ptr));
  if (glue_flag) {
    generic ret = symbol ("ret");
    list    mmxa= view_as_mmx (tl4, args, table ());
    generic back= mmx_make_list (cons (ext_id (i), mmxa));
    if (atomic_flag) back= ext_id (i);
    generic defp= Assign (Type (ret, Pointer (abbr_type_rep)), ptr);
    generic sett= Assign (Arrow (ret, symbol ("mmxt")), back);
    body= Begin (defp, sett, Return (ret));
  }
  generate (Define (fun, body));

  // Produce the routines
  while ((!nil (l)) && (!nil (r))) {
    contype_routine (ccpp_id, ccpp_type, args, car (l), car (r));
    l= cdr (l);
    r= cdr (r);
  }
}

void
extender::produce_contype (generic g, int i) {
  // cout << "Produce concrete type " << g << lf;
  if (!(is_func (g, MMX_DEFINE, 2) &&
	is_func (g[1], MMX_TYPE, 2) &&
	is_symbol (g[2])))
    error (g, "Assertion failed in produce_contype");
  else {
    generic type= g[1][2];
    if (type == MMX_CLASS) contype_produce (g[1][1], g[2], list (), i, true);
    else if (is_func (type, MMX_FUNCTION, 2) && (type[2] == MMX_CLASS)) {
      list l (type[1]);
      if (is_func (type[1], MMX_CROSS)) l= cdr (as_list (type[1]));
      contype_produce (g[1][1], g[2], l, i, false);
    }
  }
}

void
extender::produce_contype (list l, int i) {
  if (!nil (l)) {
    produce_contype (car (l), i);
    produce_contype (cdr (l), i+1);
  }
}

} // namespace mmx
