
/******************************************************************************
* MODULE     : extend.hh
* DESCRIPTION: Convert mathemagix trees to C or C++
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __EXTEND_HH
#define __EXTEND_HH
#include <symbols.hh>
#include <list.hh>
namespace mmx {

class extender {
private:
  bool   glue_flag;
  int    nr_errors;
  string preamble;
  string postamble;
  list   im_types;
  table  im_type_conv;
  list   ex_types;
  table  ex_type_conv;
  list   exports;
  table  export_env;
  list   imports;
  table  import_env;
  list   gen_imports;
  table  gen_import_env;
  list   output;
  int    max_type_args;
  list   glue_init;
  int    nr_ext_tvar;

  // names
  string  the_prefix;
  symbol  genid;
  symbol  gentype;
  symbol  gentype_rep;
  generic genmethod (generic name);
  generic geninst ();
  generic geninst (int nr);
  generic curtype (int nr);
  generic contype (generic name);
  generic contype_rep (generic name);

  generic mmx_scope (generic g);
  generic mmx_symbol (symbol s);
  generic mmx_specialize (symbol s, generic type, generic id);
  generic mmx_generic;
  generic mmx_mirror;
  symbol  ext_cv ();
  symbol  ext_id (int i);
  symbol  ext_fun (int i);
  symbol  ext_conv (int i);
  symbol  ext_tvar (int i);
  symbol  ext_arg (int i);

  // error handling, code generation, and other helper routines
  void    error (generic where, string message);
  void    warning (generic where, string message);
  void    generate (generic g);
  void    generate_init (generic g);
  void    declare (generic var, generic val, generic type);
  generic mmx_make_list (list l);

  // parsing
  generic rewrite (generic g);
  void    ex_type_decl (generic g);
  void    gen_import_decl (generic g);
  void    export_decl (generic g);
  generic add_default_routines (generic type);
  void    add_default_routines ();
  void    parse (generic prog);

  // conversions
  generic type_as_ccpp (generic g, table t= table ());
  list    type_as_ccpp (list l, table t= table ());
  generic decl_as_ccpp (generic g, bool gen_flag= false);
  list    decl_as_ccpp (list l, bool gen_flag= false);
  generic view_as_ccpp (generic expr, generic mmx_type, table t= table ());
  generic view_as_mmx (generic expr, generic mmx_type, table t);
  list    view_as_mmx (list exprs, list mmx_types, table t);

  // extension code for generic types
  generic specialize (generic g, generic to, bool virtual_flag);
  list    specialize (list l, generic to, bool virtual_flag);
  list    gentype_args_as_mmx (list l);
  void    gentype_methods (generic g);
  void    gentype_methods (list l);
  void    produce_gentype ();
  void    produce_geninst ();
  list    abstract (int nr, list args, list types);
  void    geninst_constructors (int nr, generic g, list& r);
  list    geninst_constructors (int nr, list l);
  void    geninst_methods (int nr, generic g);
  void    geninst_methods (int nr, list l);
  void    produce_geninst (int nr);

  // extension code for concrete types
  list    filter (list l, list target_l);
  list    class_to_geninst (list l, int i=1);
  list    contype_tmpl_args (list l, int kind, int i=1, int n=1);
  list    contype_cast_args (generic tp, list l, int i);
  void    contype_routine (generic n, generic t, list a, generic g, generic h);
  void    contype_produce (generic mid, generic cid, list ta, int i, bool f);
  void    produce_contype (generic g, int i);
  void    produce_contype (list l, int i);

  // generation of the glue
  bool    match_mmx_type (generic expr, generic defn, list bindings);
  int     resolve_mmx_type (generic expr, list l, int i, list bindings);
  generic build_mmx_type_sub (generic g, list bindings);
  list    build_mmx_type (list l, list bindings);
  generic build_mmx_type (generic g, list bindings);
  void    produce_glue_type (generic g, int i, bool prototype);
  void    produce_glue_type (list l, int i, bool prototype);
  list    glue_template_args (list l, int kind, int i=1, int n=1);
  list    glue_convert_args (list l, generic x, table t, int i);
  void    produce_glue_routine (generic g, int i, list bindings);
  void    produce_glue_routine (list l, int i);
  void    produce_glue ();

public:
  inline extender (string prefix, bool glue_flag2):
    glue_flag (glue_flag2),
    nr_errors (0),
    max_type_args (0),
    nr_ext_tvar (0),
    the_prefix (prefix),
    genid ("TID"),
    gentype (the_prefix * "_type"),
    gentype_rep (the_prefix * "_type_rep"),
    mmx_generic (mmx_symbol ("generic")),
    mmx_mirror (mmx_symbol ("mirror")) {}
  inline ~extender () {}

  void produce (ostream& out, generic prog);

  friend bool extension_code (ostream& out, string prefix, generic p, bool g);
};

inline symbol operator * (symbol s1, symbol s2) {
  return symbol (as_string (S (s1)) * as_string (S (s2))); }
void nice_print (generic text, list l);
void nice_print (symbol text, list l);

generic get_decl_var (generic g);
generic get_decl_type (generic g);
generic get_decl_val (generic g);
generic make_struct (generic name, list args, list inherit= list ());
generic make_blackbox_struct (generic name, list extra= list ());
generic ccpp_type_protect (generic type);
symbol  parameter (int i);
generic name_parameters (generic name);

}
#endif // __EXTEND_HH
